@@ -119,12 +119,12 @@ def walk(reg, keyName):
 
 
 def main():
-    # Init the example's logger theme
-    logger.init()
     print(version.BANNER)
 
     parser = argparse.ArgumentParser(add_help = True, description = "Reads data from registry hives.")
 
+    parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
+    parser.add_argument('-ts', action='store_true', help='Adds timestamp to every logging output')
     parser.add_argument('hive', action='store', help='registry hive to open')
     subparsers = parser.add_subparsers(help='actions', dest='action')
     # A enum_key command
@@ -153,8 +153,10 @@ def main():
         sys.exit(1)
 
     options = parser.parse_args()
+    # Init the example's logger theme
+    logger.init(options.ts, options.debug)
 
-    reg = winregistry.Registry(options.hive)
+    reg = winregistry.get_registry_parser(options.hive)
 
     if options.action.upper() == 'ENUM_KEY':
         print("[%s]" % options.name)
